// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MefCommon;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;

namespace MefLabMain
{
    
    public partial class MainForm : Form
    {
        private readonly string _extensionDir = "ExtModules";

        [ImportMany]
        public Lazy<IMainFormContract, IDictionary<string, object>>[] ImportedMainFormContracts { get; set; }

        private CompositionContainer _container;

        private CompositionContainer GetContainerFromDirectory()
        {
            var catalog = new AggregateCatalog();
            var thisAssembly =
                new AssemblyCatalog(
                    System.Reflection.Assembly.GetExecutingAssembly());
            catalog.Catalogs.Add(thisAssembly);

            catalog.Catalogs.Add(
                new DirectoryCatalog(_extensionDir));

            var container = new CompositionContainer(catalog);
            return container;
        }

        private bool Compose()
        {
            _container = GetContainerFromDirectory();

            try
            {
                _container.ComposeParts(this);
            }

            catch (CompositionException compException)
            {
                MessageBox.Show(compException.ToString());
                return false;
            }

            return true;
        }

        public MainForm()
        {
            InitializeComponent();
            bool successfulCompose = Compose();
            if (!successfulCompose)
            {
                this.Close();
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            foreach (var export in this.ImportedMainFormContracts)
            {
                var exportedMenuText = export.Metadata["MenuText"] as string;
                if (String.IsNullOrEmpty(exportedMenuText))
                {
                    return;
                }
                ToolStripItem menuItem =
                    modulesToolStripMenuItem.DropDownItems.Add(exportedMenuText);
                menuItem.Click += new System.EventHandler(this.LaunchModule_Click);
            }
        }

        private void LaunchModule_Click(object sender, EventArgs e)
        {
            ToolStripItem thisItem = sender as ToolStripItem;
            if (thisItem == null) { return; }

            string thisItemTitle = thisItem.Text;

            foreach (var export in this.ImportedMainFormContracts)
            {
                string menuTitle = export.Metadata["MenuText"] as string;
                if (String.IsNullOrEmpty(menuTitle))
                {
                    return;
                }
                if (menuTitle == thisItemTitle)
                {
                    Form frm = export.Value as Form;
                    if (frm == null) { return; }
                    frm.Show(this);
                    return;
                }
            }
        }
    }
}