// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using MefCommon;
using System.ComponentModel.Composition;

namespace MefEmployeeModule
{
    [Export(typeof(IMainFormContract))]
    [ExportMetadata("Name", "Employee Pane")]
    [ExportMetadata("MenuText", "&Employees")]
    public partial class EmployeeMaintenance : Form, IMainFormContract
    {
        private readonly List<Employee> employees = new List<Employee>();
        private CompanyInfo CompanyInfo { get; set; }

        [ImportMany]
        public Lazy<ICmdButtonInfo>[] ImportedButtons
        {
            get;
            set;
        }

        private void CreateButtons()
        {
            int left = 10;

            foreach (var importedButton in this.ImportedButtons)
            {
                var btnInfo = importedButton.Value;

                btnInfo.CompanyInfo = this.CompanyInfo;
                btnInfo.CompanyInfo.EmployeeListChanged +=
                    new EventHandler(CompanyInfo_EmployeeListChanged);
                Button btn = btnInfo.CommandButton;

                this.ButtonsPanel.Controls.Add(btn);
                btn.Left = left;
                left += btn.Width;
                left += 20;
            }
        }

        public string MenuItemText
        {
            get { return "&Employees"; }
        }

        public string SubFormTitle
        {
            get { return "Employee Pane"; }
        }

        public EmployeeMaintenance()
        {
            this.CompanyInfo = new CompanyInfo();
            InitializeComponent();
        }

        private void EmployeeMaintenance_Load(object sender, EventArgs e)
        {
            GetAllEmployees();
            CreateButtons();
        }

        private void empDataGridView_SelectionChanged(object sender, EventArgs e)
        {
            if (empDataGridView.SelectedRows.Count > 0)
            {
                Employee selectedEmployee
                    = (Employee)empDataGridView.SelectedRows[0].DataBoundItem;
                this.CompanyInfo.SelectedEmployee = selectedEmployee;
            }
            else
            {
                this.CompanyInfo.SelectedEmployee = null;
            }
        }

        /// <summary>
        /// Simulates getting data from a source
        /// </summary>
        private void GetAllEmployees()
        {
            var emp1 = new Employee
                (1, "George", "Washington", "123 Main",
                 "Mt Vernon", "VA", "20853", new DateTime(2006, 1, 1));
            var emp2 = new Employee
                (2, "Tom", "Jefferson", "456 Elm",
                 "Monticello", "VA", "30111", new DateTime(2006, 1, 1));
            var emp3 = new Employee
                (3, "John", "Adams", "789 Maple",
                 "Rockville", "MD", "20853", new DateTime(2006, 1, 1));
            employees.Add(emp1);
            employees.Add(emp2);
            employees.Add(emp3);
            this.CompanyInfo.Employees = employees;
            empDataGridView.DataSource = employees;
        }

        void CompanyInfo_EmployeeListChanged(object sender, EventArgs e)
        {
            empDataGridView.DataSource = null;
            empDataGridView.DataSource = this.CompanyInfo.Employees;
        }
    }
}